import datetime

import requests
from bs4 import BeautifulSoup

base = "https://www.calendarioslaborales.com/"
headers = {
    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_3) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.132 Safari/537.36 "
}


def get_provinces() -> list:
    page = requests.get(base, headers=headers)
    soup = BeautifulSoup(page.content, "html.parser")
    provinces = soup.findAll(
        "a", title=lambda value: value and value.startswith("Calendario laboral")
    )
    result = []
    for province in provinces:
        calendar = province.get("href").split("/")[-1].split(".")[0].split("-")[2:]
        calendar.pop()
        result.append("-".join(calendar))
    return result


class Province:
    def __init__(self, name, year=None):
        self.name = name
        self.year = datetime.datetime.now().year if year is None else year
        self._page = requests.get(
            f"{base}calendario-laboral-{name}-{self.year}.htm", headers=headers
        )
        self._soup = BeautifulSoup(self._page.content, "html.parser")
        self.__check_errors()

    def __check_errors(self):
        if self._soup.find(text="Warning") is not None:
            raise Exception("There are no records for the specified year.")
        if self._soup.find(text="Error:") is not None:
            raise Exception(
                f"The name is not in the list, try one of these: {', '.join(get_provinces())}."
            )

    def __tour_months(self, holiday):
        result = []
        for m in months():
            table_id = f"mes{m.title()}"
            table = self._soup.find(id=table_id)
            holidays = table.findAll("td", {"class": f"cajaFestivo{holiday}"})
            for h in holidays:
                day = int(h.get_text())
                month = months().index(m) + 1
                date = datetime.date(day=day, month=month, year=self.year)
                result.append(date)
        return result

    def national_holidays(self) -> list:
        return self.__tour_months("N")

    def regional_holidays(self) -> list:
        return self.__tour_months("R")

    def local_holidays(self) -> list:
        return self.__tour_months("P")

    def holidays(self):
        return {
            "national_holidays": self.national_holidays(),
            "regional_holidays": self.regional_holidays(),
            "local_holidays": self.local_holidays(),
        }


def months() -> tuple:
    return (
        "enero",
        "febrero",
        "marzo",
        "abril",
        "mayo",
        "junio",
        "julio",
        "agosto",
        "septiembre",
        "octubre",
        "noviembre",
        "diciembre",
    )



import datetime

from holidays_es import get_provinces, Province


def test_provinces_count():
    """
    In the list of provinces you will also find municipalities,
    this is by https://www.calendarioslaborales.com/.
    """
    provinces = get_provinces()
    assert len(provinces) == 61


def test_types_holidays():
    """we have national, regional and local festivals."""
    holidays = Province(name="valencia").holidays()
    assert len(holidays) == 3


def test_national_holidays():
    """All the provinces have the same national holidays..."""
    valencia = Province(name="valencia").national_holidays()
    barcelona = Province(name="barcelona").national_holidays()
    assert len(valencia) == len(barcelona)


def test_regional_holidays():
    """In Valencia we have several regional festivals."""
    valencia = Province(name="valencia").regional_holidays()
    assert len(valencia) > 0


def test_local_holidays_and_current_year():
    """If we don't specify a year it will choose the current year."""
    valencia = Province(name="valencia").local_holidays()
    current_year = datetime.datetime.now().year
    assert valencia[0].year == current_year


def test_year_holidays():
    """Check that you can filter for holidays since 2016."""
    for year in range(2016, 2020):
        valencia = Province(name="valencia", year=year).national_holidays()
        assert valencia[0].year == year



import setuptools

setuptools.setup(
    name="holidays_es",
    version="1.0.1",
    url="https://github.com/rsolanoweb/holidays-es",
    author="Rubén Solano",
    author_email="rubensoljim@gmail.com",
    description="Public holidays in Spain from 2006 to now.",
    long_description=open("README.rst").read(),
    packages=setuptools.find_packages(),
    install_requires=[],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
)
